import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.Random;

import javax.swing.JComponent;

public class GameField extends JComponent
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -4610376013123566082L;
	public final static Integer width = 20;
	public final static Integer height = 20;
	private Snake snake;
	private int[][] field;
	public int[] foodPosition;
	private FoodImages foodType;

	public void prepareField(Integer fieldIndex, Snake snake)
	{
		this.snake = snake;
		field = new int[width][height];
		
		String[] fieldString = Fields.fields[fieldIndex];
		for(int y=0;y<fieldString.length;y++)
			for(int x=0;x<fieldString[y].length();x++)
				if(fieldString[y].charAt(x)!=' ')
					field[x][y] = 1;
				else
					field[x][y] = 0;
		foodPosition = new int[2];
		addFood();
				
		repaint();
	}

	public void paintComponent(Graphics graphics)
	{
		super.paintComponent(graphics);
		Graphics2D g = (Graphics2D) graphics.create();
		g.setBackground(Color.WHITE);
		g.clearRect(0, 0, width*25, height*25);

		if (field == null)
			return;

		for (int x = 0; x < width; x++)
			for (int y = 0; y < height; y++)
				if (field[x][y] == 1)
					drawWall(g, x*25, y*25);
		drawFood(g);
		snake.draw(g);
		g.dispose();
	}

	public Dimension getMinimumSize()
	{
		return new Dimension(width * 25, height * 25);
	}

	public Dimension getPreferredSize()
	{
		return getMinimumSize();
	}

	private void drawWall(Graphics2D g, Integer x, Integer y)
	{
		g.drawImage(Window.image.get("Wall"), x, y, Color.white, null);
	}

	private void drawFood(Graphics2D g)
	{
		g.drawImage(Window.image.get(foodType.toString()), foodPosition[0]*25, foodPosition[1]*25, Color.white, null);
	}

	public int getField(int x, int y)
	{
		return field[x][y];
	}

	public void addFood()
	{
		Random r = new Random();
		int x, y;
		while (true)
		{
			x = r.nextInt(width);
			y = r.nextInt(height);
			if (getField(x, y) == 1)
				continue;
			for (SnakePart p : snake.getParts())
				if (p.x == x && p.y == y)
					continue;
			break;
		}
		foodPosition[0] = x;
		foodPosition[1] = y;
		foodType = FoodImages.values()[r.nextInt(FoodImages.values().length)];
	}
}
