import java.awt.Color;
import java.awt.Graphics2D;
import java.util.Vector;

public class Snake
{
	private Vector<SnakePart> parts;
	private Integer directionHorizontal;
	private Integer directionVertical;
	
	public Snake(Integer length)
	{
		parts = new Vector<SnakePart>();
		for(int i=1;i<=length;i++)
			parts.add(new SnakePart(length-i, 1));
		setDirection(1);
	}
	
	public Vector<SnakePart> getParts()
	{
		return parts;
	}
	
	public void draw(Graphics2D g)
	{
		for(int i=0;i<parts.size();i++)
			drawPart(i, g);
	}
	
	public Boolean move(Boolean eat)
	{
		SnakePart head = head();
		Integer newX = head.x + directionHorizontal;
		Integer newY = head.y + directionVertical;
		if(newX<0)
			newX = GameField.width-1;
		if(newY<0)
			newY = GameField.height-1;
		if(newX>=GameField.width)
			newX = 0;
		if(newY>=GameField.height)
			newY = 0;
		for(SnakePart p:parts)
			if(p.x==newX && p.y==newY)
				return false;
		parts.add(0, new SnakePart(newX, newY));
		if(!eat)
			parts.remove(parts.lastElement());
		return true;
	}
	
	public SnakePart head()
	{
		return parts.get(0);
	}
	
	public void setDirection(Integer d)
	{
		switch(d)
		{
		case 0:
			directionHorizontal = 0;
			directionVertical = -1;
			break;
		case 1:
			directionHorizontal = 1;
			directionVertical = 0;
			break;
		case 2:
			directionHorizontal = 0;
			directionVertical = 1;
			break;
		case 3:
			directionHorizontal = -1;
			directionVertical = 0;
			break;
		}
	}
	
	private void drawPart(Integer i, Graphics2D g)
	{
		SnakeImages img = null;
		if(i==0) //head
		{
			switch(directionHorizontal)
			{
			case -1:
				img = SnakeImages.Head3;
				break;
			case 1:
				img = SnakeImages.Head1;
				break;
			}
			switch(directionVertical)
			{
			case -1:
				img = SnakeImages.Head0;
				break;
			case 1:
				img = SnakeImages.Head2;
				break;
			}
		}
		else if(i==(parts.size()-1)) //tail
		{
			switch(parts.get(i-1).x - parts.get(i).x)
			{
			case -1:
				img = SnakeImages.Tail3;
				break;
			case 1:
				img = SnakeImages.Tail1;
				break;
			}
			switch(parts.get(i-1).y - parts.get(i).y)
			{
			case -1:
				img = SnakeImages.Tail0;
				break;
			case 1:
				img = SnakeImages.Tail2;
				break;
			}
		}
		else
		{
			Integer diffX = parts.get(i-1).x - parts.get(i+1).x;
			Integer diffY = parts.get(i-1).y - parts.get(i+1).y;
			Integer diff = parts.get(i-1).x - parts.get(i).x; 
			if((diffX==-1 && diffY==-1 && diff==0) ||
					(diffX==1 && diffY==1 && diff==1))
				img = SnakeImages.Corner0;
			else if((diffX==-1 && diffY==-1 && diff==-1) ||
					(diffX==1 && diffY==1 && diff==0))
				img = SnakeImages.Corner2;
			else if((diffX==1 && diffY==-1 && diff==1) ||
					(diffX==-1 && diffY==1 && diff==0))
				img = SnakeImages.Corner1;
			else if((diffX==1 && diffY==-1 && diff==0) ||
					(diffX==-1 && diffY==1 && diff==-1))
				img = SnakeImages.Corner3;
			else if(diffX==0)
				img = SnakeImages.BodyV;
			else if(diffY==0)
				img = SnakeImages.BodyH;
		}
		drawImage(g, parts.get(i).x*25, parts.get(i).y*25, img);
	}
	
	private void drawImage(Graphics2D g, Integer x, Integer y, SnakeImages img)
	{
		if(img==null)
			return;
		g.drawImage(Window.image.get(img.toString()), x, y, Color.WHITE, null);
	}
}