import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

public class Window extends JFrame implements ActionListener, KeyListener
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1649704890423103717L;
	public static Image image;
	private Integer level = new Integer(1);
	private Integer fieldIndex = new Integer(0);
	
	private GameField gameField;
	private GameFieldPreview gameFieldPreview;
	private Counter scoreField;
	private Game game;
	private Thread gameThread;
	
	private JButton prevLevel, nextLevel, prevField, nextField, newGame;
	private Counter levelField;
	
	public Window()
	{
		add(gamePanel(), BorderLayout.CENTER);
		add(controlPanel(), BorderLayout.LINE_END);
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		pack();
		setTitle("Zielony w w czarne paski");
		setResizable(false);
		
		setVisible(true);
		setFocusable(true);
		requestFocus();
		addKeyListener(this);
		
	}
	
	private JPanel gamePanel()
	{
		JPanel gamePanel = new JPanel();
        gamePanel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder(""),
                BorderFactory.createEmptyBorder(0,0,0,0)));
        gamePanel.add(this.gameField = new GameField());
        gamePanel.addKeyListener(this);
        gamePanel.setBackground(Color.WHITE);
        
        return gamePanel;
    }
	
	private JPanel controlPanel()
	{
		JPanel controlPanel = new JPanel();
		JLabel scoreLabel, levelLabel, fieldLabel, dedication;
		
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
		controlPanel.add(Box.createRigidArea(new Dimension(0,10)));
		controlPanel.add(scoreLabel = new JLabel("Punkty:"));
		scoreLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		controlPanel.add(scoreField = new Counter(6, 0));
		controlPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		controlPanel.add(levelLabel = new JLabel("Poziom:"));
		levelLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		controlPanel.add(levelSelect());
		controlPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		controlPanel.add(fieldLabel = new JLabel("Plansza:"));
		fieldLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		controlPanel.add(fieldSelect());
		controlPanel.add(Box.createRigidArea(new Dimension(0,10)));
		controlPanel.add(newGame = new JButton("Nowa gra"));
		newGame.setAlignmentX(Component.CENTER_ALIGNMENT);
		newGame.addActionListener(this);
		controlPanel.add(Box.createRigidArea(new Dimension(0,10)));
		controlPanel.add(dedication = new JLabel("<html><i>Z dedykacj dla A.,<br/>ktra zapytaa, czy<br/>umiabym napisa gr, np. wa.</i></html>"));
		dedication.setAlignmentX(Component.LEFT_ALIGNMENT);
		controlPanel.add(Box.createVerticalGlue());
		return controlPanel;
	}
	
	private JPanel levelSelect()
	{
		JPanel levelSelect = new JPanel();
		levelSelect.setLayout(new BoxLayout(levelSelect, BoxLayout.X_AXIS));
		levelSelect.add(prevLevel = new JButton(new ImageIcon(image.get("digits/prev"))));
		prevLevel.addActionListener(this);
		prevLevel.setMaximumSize(new Dimension(25,25));
		prevLevel.setPreferredSize(new Dimension(25,25));
		levelSelect.add(Box.createRigidArea(new Dimension(10, 10)));
		levelSelect.add(levelField = new Counter(2, level));
		levelSelect.add(Box.createRigidArea(new Dimension(10, 10)));
		levelSelect.add(nextLevel = new JButton(new ImageIcon(image.get("digits/next"))));
		nextLevel.addActionListener(this);
		nextLevel.setMaximumSize(new Dimension(25,25));
		nextLevel.setPreferredSize(new Dimension(25,25));
		return levelSelect;
	}
	
	private JPanel fieldSelect()
	{
		JPanel fieldSelect = new JPanel();
		fieldSelect.setLayout(new BoxLayout(fieldSelect, BoxLayout.X_AXIS));
		fieldSelect.add(prevField = new JButton(new ImageIcon(image.get("digits/prev"))));
		prevField.addActionListener(this);
		prevField.setMaximumSize(new Dimension(25,25));
		prevField.setPreferredSize(new Dimension(25,25));
		fieldSelect.add(Box.createRigidArea(new Dimension(0, 10)));
		fieldSelect.add(gameFieldPreview = new GameFieldPreview(fieldIndex));
		gameFieldPreview.setAlignmentY(Component.CENTER_ALIGNMENT);
		fieldSelect.add(Box.createRigidArea(new Dimension(0, 10)));
		fieldSelect.add(nextField = new JButton(new ImageIcon(image.get("digits/next"))));
		nextField.addActionListener(this);
		nextField.setMaximumSize(new Dimension(25,25));
		nextField.setPreferredSize(new Dimension(25,25));
		return fieldSelect;
	}
	
	public void actionPerformed(ActionEvent evt)
	{
		requestFocus();
		if(evt.getSource()==prevLevel)
		{
			level--;
			if(level<1)
				level = 1;
			levelField.setValue(level);
			levelField.repaint();
		}
		if(evt.getSource()==nextLevel)
		{
			level++;
			if(level>10)
				level = 10;
			levelField.setValue(level);
			levelField.repaint();
		}
		if(evt.getSource()==prevField)
		{
			fieldIndex--;
			if(fieldIndex<0)
				fieldIndex = 0;
			gameFieldPreview.setFieldIndex(fieldIndex);
			gameFieldPreview.repaint();
		}
		if(evt.getSource()==nextField)
		{
			fieldIndex++;
			if(fieldIndex>=Fields.fields.length)
				fieldIndex = Fields.fields.length-1;
			gameFieldPreview.setFieldIndex(fieldIndex);
			gameFieldPreview.repaint();
		}
		if(evt.getSource()==newGame)
		{
			if(gameThread != null && gameThread.isAlive())
				gameThread.interrupt();
			game = new Game(gameField, level, fieldIndex, scoreField);
			gameThread = new Thread(game);
			gameThread.start();
		}
	}

	public void keyPressed(KeyEvent arg0)
	{
		if(gameThread==null || !gameThread.isAlive())
			return;
		switch(arg0.getKeyCode())
		{
		case KeyEvent.VK_DOWN :
			game.snake.setDirection(2);
			break;
		case KeyEvent.VK_UP:
			game.snake.setDirection(0);
			break;
		case KeyEvent.VK_LEFT:
			game.snake.setDirection(3);
			break;
		case KeyEvent.VK_RIGHT:
			game.snake.setDirection(1);
			break;
		}
	}

	public void keyReleased(KeyEvent arg0)
	{}

	public void keyTyped(KeyEvent arg0)
	{}
	
	public static void main(String[] args)
	{
		Window.image = new Image();
		try
		{
			SwingUtilities.invokeAndWait(
					new Runnable()
					{
						public void run()
						{
							new Window();
						}
					}
			);
		}
		catch(Exception e)
		{
			System.err.println("Nie mog otworzy okna.");
			System.exit(-1);
		}
	}
}
